--[[Basis-Script zum Ändern von FillTypes.

2020 Oma Tana (Eva Marbach)
Version 0.8

In diesem Script wird eine einzelne Art von Filltypes in eine einzelne andere Filltyp-Art verwandelt,
z.B. diverse Grasarten zu Silage oder diverse Ölfrüchte (z.B. Raps, Sojabohnen) zu Benzin

Was für Werte sollen verarbeitet werden:

inputFillTypes                      mehrere Fuelltypen durch Leerschrrit trennen
outputFillType                      hier ist nur ein Fuelltyp moeglich
quantityPerMinute                   in Liter
transformRatioPercent               in Prozent. Standard ist 100
capacityPerFillType                 in Liter. Achtung! Kapazitätslimits werden (noch) nicht überprüft
productionObjectName                muss identisch sein mit loadingStation.stationName
displayIn                           hier Base-Node aus der i3d eintragen. Das Objekt in der i3D muss WeightDisplayIn heißen
displayOut                          hier Base-Node aus der i3d eintragen. Das Objekt in der i3D muss WeightDisplayOut heißen

Syntax-Beispiel zur Nutzung in der xml-Datei:
<inputFillTypes>chaff grass_windrow dryGrass_windrow straw</inputFillTypes>
<outputFillType>silage</outputFillType>
<quantityPerMinute>1000</quantityPerMinute>
<transformRatioPercent>100</transformRatioPercent>
<capacityPerFillType>1000000</capacityPerFillType>
<productionObjectName>FermenterSilo</productionObjectName>
<displayIn baseNode="6|0" />
<displayOut baseNode="6|1" />

--]]

fillTypeChanger = {}
cProduction ={}
fillTypesIn = {}
fillTypeChanger.In = {}
fillTypeChanger.Prod = {}

local szOutputLiters = 0
local iUpdateTime = 60000
iUpdateInterval = 60000
iAnzahlMinuten = 0

-- Öffnen der ModDateien

local modName = g_currentModName

local modDesc = loadXMLFile("modDesc", g_currentModDirectory .. "modDesc.xml")
local modVersion = getXMLString(modDesc, "modDesc.version")
local modFileName = getXMLString(modDesc, "modDesc.storeItems(0).storeItem(0)#xmlFilename")
local modFile = loadXMLFile("modFile", g_currentModDirectory .. modFileName)

-- Auslesen der Parameter der ModDatei

local szProductionObjectName = getXMLString(modFile, "placeable.productionObjectName")
local szInputFillTypes = getXMLString(modFile, "placeable.inputFillTypes")
local szOutputFillType = getXMLString(modFile, "placeable.outputFillType")
local iQuantityPerMinute = getXMLInt(modFile, "placeable.quantityPerMinute")
local iTransformRatioPercent = getXMLInt(modFile, "placeable.transformRatioPercent")
local iCapacityPerFillType = getXMLInt(modFile, "placeable.capacityPerFillType")
local szDisplayInBaseNode = getXMLString(modFile, "placeable.displayIn#baseNode")
local szDisplayOutBaseNode = getXMLString(modFile, "placeable.displayOut#baseNode")

--[[
print(modVersion)
print(modFileName)
print(szProductionObjectName)
print(szInputFillTypes)
print(szOutputFillType)
print(iQuantityPerMinute)
print(iCapacityPerFillType)
print(szDisplayInBaseNode)
print(szDisplayOutBaseNode)
--]]

-- FillType-Liste wird zerlegt. Split an Leerschritten
local i = 1
for substring in szInputFillTypes:gmatch("%S+") do
    -- print(substring)
    fillTypesIn[i] = substring
    i = i+1
end
local iAnzahlFillTypes = i - 1
-- print("Es gibt so viele FillTypes: ", iAnzahlFillTypes)

if iTransformRatioPercent == nil then
	iTransformRatioPercent = 100
end

if szDisplayOutBaseNode == "" then
	iDisplayQuantities = 0
else
	iDisplayQuantities = 1
end


-- Dies ist ganz wichtig, damit die function funktioniert
addModEventListener(fillTypeChanger);

-- print("Ende des Programms")

function fillTypeChanger:update(dt)

    local iPlaceableCounter = 0
    local szFillTypesIn = ""
    local szFillTypesInStorage = ""
    local szFillTypeNameIn = ""
    local szFillTypeNameInStorage = ""
    local szFillTypeOutStorage = ""
    local szActivePlaceable = ""
    local iRemainderQuantity = 0
    local iChangerQuantity = 0
	local iChangerQuantityOut = 0
	local iActiveFillTypeQuantity = 0
	local iSummeIn = 0
    local iSummeOut = 0
    fillTypeChanger.FillTypeNameIn = ""
    fillTypeChanger.summeIn = 0

    iUpdateTime = iUpdateTime + (dt * g_currentMission.loadingScreen.missionInfo.timeScale)
    if iUpdateTime >= iUpdateInterval   then
        iUpdateTime = iUpdateTime - iUpdateInterval
        iAnzahlMinuten = iAnzahlMinuten +1
        -- print("Anzahl Minuten:" .. iAnzahlMinuten)
		iRemainderQuantity = iQuantityPerMinute

        for iPlaceableCounter=1, #g_currentMission.placeables do
            if g_currentMission.placeables[iPlaceableCounter] ~= nil then
                local szActivePlaceable = g_currentMission.placeables[iPlaceableCounter]
				
                if szActivePlaceable.loadingStation ~= nil and szActivePlaceable.loadingStation.stationName ~= nil  then
				
					-- print("nodeId von " .. szActivePlaceable.loadingStation.stationName .. ": " .. szActivePlaceable.nodeId)
				
                    if szActivePlaceable.loadingStation.stationName == szProductionObjectName then
                        -- Hier habe ich das gesuchte Placeable gefunden                    
                        -- print("Gesuchtes Placeable: " .. szProductionObjectName)

                        for szFillTypesInStorage,isAccepted in pairs(szActivePlaceable.storages[1].fillTypes) do
                            szFillTypeNameInStorage = g_fillTypeManager.indexToName[szFillTypesInStorage];
                            -- print(szFillTypeNameInStorage)
                            -- Vorhandene Output-Menge identifizieren
                            if szFillTypeNameInStorage == string.upper(szOutputFillType) then    
                                iSummeOut = szActivePlaceable.storages[1].fillLevels[szFillTypesInStorage]
                                szFillTypeOutStorage = szFillTypesInStorage
                            end

                            for i,szFillTypesIn,isAccepted in pairs(fillTypesIn) do
                                if szFillTypeNameInStorage == string.upper(szFillTypesIn) then    
                                    -- Hier finde ich die Füllmenge der Input-Filltypes
                                    if szActivePlaceable.storages[1].fillLevels[szFillTypesInStorage] > 0 then
                                        -- Abarbeitung aller Input-Mengen, unter Berücksichtigung der minuetlichen Kapazitaet
										iActiveFillTypeQuantity = szActivePlaceable.storages[1].fillLevels[szFillTypesInStorage]
                                        -- print("Fuellstand In " .. szFillTypeNameInStorage .. ": " .. iActiveFillTypeQuantity)
										if iRemainderQuantity > 0 then
											if iActiveFillTypeQuantity >= iRemainderQuantity then
                                                iChangerQuantity = iChangerQuantity + iRemainderQuantity
                                                iActiveFillTypeQuantity = iActiveFillTypeQuantity - iRemainderQuantity
                                                iRemainderQuantity = 0
											else
											    iChangerQuantity = iChangerQuantity + iActiveFillTypeQuantity
												iRemainderQuantity = iRemainderQuantity - iActiveFillTypeQuantity
												iActiveFillTypeQuantity = 0
											end
										    -- print("ChangeMenge: " .. iChangerQuantity .. " Remainder: " .. iRemainderQuantity)

										end
										-- Summe wird fuer Anzeige gebraucht
                                        iSummeIn = iSummeIn + iActiveFillTypeQuantity
                                        -- print("Summe In: " .. iSummeIn)
                                        -- Setzen der neuen Input-FillLevels                                        
                                        szActivePlaceable.storages[1].fillLevels[szFillTypesInStorage] = iActiveFillTypeQuantity
                                    end
                                end
                            end
                        end
                        -- Input-Mengen auf Output-Filltype umbuchen             
						iChangerQuantityOut = iChangerQuantity * iTransformRatioPercent/100
                        iSummeOut = iSummeOut + iChangerQuantityOut
                        -- print("Output " .. szOutputFillType .. ": " .. iSummeOut)
                        szActivePlaceable.storages[1].fillLevels[szFillTypeOutStorage] = iSummeOut
						
						if iDisplayQuantities == 1 then
                            local oBaseNodeIn = I3DUtil.indexToObject(szActivePlaceable.nodeId, szDisplayInBaseNode)
                            local oBaseNodeOut = I3DUtil.indexToObject(szActivePlaceable.nodeId, szDisplayOutBaseNode)
                            I3DUtil.setNumberShaderByValue(oBaseNodeIn, iSummeIn, 0, true)
                            I3DUtil.setNumberShaderByValue(oBaseNodeOut, iSummeOut, 0, true)

                        end
                    end
                end
            end
        end


    end


end









